@@ -26,6 +26,7 @@ from pdm.cli.utils import (
     find_importable_files,
     format_lockfile,
     format_resolution_impossible,
+    get_pep582_path,
     merge_dictionary,
     save_version_specifiers,
     set_env_in_reg,
@@ -45,8 +46,6 @@ from pdm.project import Project
 from pdm.resolver import resolve
 from pdm.utils import cd, normalize_name
 
-PEP582_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)), "pep582")
-
 
 def do_lock(
     project: Project,
@@ -72,7 +71,8 @@ def do_lock(
                 candidate.requires_python = python_requires
                 mapping[candidate.identify()] = candidate
                 dependencies[candidate.dep_key] = list(map(parse_requirement, reqs))
-            fetch_hashes(repo, mapping)
+            with project.core.ui.logging("lock"):
+                fetch_hashes(repo, mapping)
             lockfile = format_lockfile(project, mapping, dependencies)
         project.write_lockfile(lockfile)
         return mapping
@@ -98,6 +98,7 @@ def do_lock(
                     project.environment.python_requires,
                     resolve_max_rounds,
                 )
+                spin.update("Fetching hashes for resolved packages...")
                 fetch_hashes(provider.repository, mapping)
         except ResolutionTooDeep:
             ui.echo(f"{termui.Emoji.LOCK} Lock failed", err=True)
@@ -133,7 +134,7 @@ def resolve_candidates_from_lockfile(
         if not req.marker or req.marker.evaluate(project.environment.marker_environment)
     ]
     with ui.logging("install-resolve"):
-        with ui.open_spinner("Resolving packages from lockfile..."):
+        with ui.open_spinner("Resolving packages from lockfile...") as spinner:
             reporter = BaseReporter()
             provider = project.get_provider(for_install=True)
             resolver: Resolver = project.core.resolver_class(provider, reporter)
@@ -143,6 +144,7 @@ def resolve_candidates_from_lockfile(
                 project.environment.python_requires,
                 resolve_max_rounds,
             )
+            spinner.update("Fetching hashes for resolved packages...")
             fetch_hashes(provider.repository, mapping)
     return mapping
 
@@ -574,10 +576,9 @@ def do_use(
         python = python.strip()
 
     def version_matcher(py_version: PythonInfo) -> bool:
-        return (
+        return py_version.valid and (
             ignore_requires_python
-            or py_version.valid
-            and project.python_requires.contains(str(py_version.version), True)
+            or project.python_requires.contains(str(py_version.version), True)
         )
 
     if not project.cache_dir.exists():
@@ -614,7 +615,8 @@ def do_use(
         if not matching_interpreters:
             project.core.ui.echo("Interpreters found but not matching:", err=True)
             for py in found_interpreters:
-                project.core.ui.echo(f"  - {py.path} ({py.identifier})", err=True)
+                info = py.identifier if py.valid else "Invalid"
+                project.core.ui.echo(f"  - {py.path} ({info})", err=True)
             raise NoPythonVersion(
                 "No python is found meeting the requirement "
                 f"[success]python {str(project.python_requires)}[/]"
@@ -752,13 +754,16 @@ def ask_for_import(project: Project) -> None:
     do_import(project, str(filepath), key)
 
 
-def print_pep582_command(ui: termui.UI, shell: str = "AUTO") -> None:
+def print_pep582_command(project: Project, shell: str = "AUTO") -> None:
     """Print the export PYTHONPATH line to be evaluated by the shell."""
     import shellingham
 
+    pep582_path = get_pep582_path(project)
+    ui = project.core.ui
+
     if os.name == "nt":
         try:
-            set_env_in_reg("PYTHONPATH", PEP582_PATH)
+            set_env_in_reg("PYTHONPATH", pep582_path)
         except PermissionError:
             ui.echo(
                 "Permission denied, please run the terminal as administrator.",
@@ -771,7 +776,7 @@ def print_pep582_command(ui: termui.UI, shell: str = "AUTO") -> None:
             style="success",
         )
         return
-    lib_path = PEP582_PATH.replace("'", "\\'")
+    lib_path = pep582_path.replace("'", "\\'")
     if shell == "AUTO":
         shell = shellingham.detect_shell()[0]
     shell = shell.lower()
