@@ -16,7 +16,7 @@ from pdm.cli.utils import Package, build_dependency_graph
 from pdm.compat import Distribution, importlib_metadata
 from pdm.models.environment import BareEnvironment, WorkingSet
 from pdm.project import Project
-from pdm.utils import normalize_name
+from pdm.utils import is_in_zipapp, normalize_name
 
 PDM_REPO = "https://github.com/pdm-project/pdm"
 
@@ -35,10 +35,18 @@ def list_distributions(plugin_only: bool = False) -> list[Distribution]:
     return sorted(result, key=lambda d: d.metadata["Name"] or "UNKNOWN")
 
 
-def run_pip(project: Project, args: list[str]) -> bytes:
+def run_pip(project: Project, args: list[str]) -> subprocess.CompletedProcess[str]:
     env = BareEnvironment(project)
     project.environment = env
-    return subprocess.check_output(env.pip_command + args, stderr=subprocess.STDOUT)
+    run_args = env.pip_command + args
+
+    return subprocess.run(
+        run_args,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+        check=True,
+        text=True,
+    )
 
 
 class Command(BaseCommand):
@@ -59,9 +67,10 @@ class Command(BaseCommand):
     def add_arguments(self, parser: argparse.ArgumentParser) -> None:
         subparsers = parser.add_subparsers(title="Sub commands")
         ListCommand.register_to(subparsers)
-        AddCommand.register_to(subparsers)
-        RemoveCommand.register_to(subparsers)
-        UpdateCommand.register_to(subparsers)
+        if not is_in_zipapp():
+            AddCommand.register_to(subparsers)
+            RemoveCommand.register_to(subparsers)
+            UpdateCommand.register_to(subparsers)
         parser.set_defaults(search_parent=False)
         self.parser = parser
 
@@ -131,7 +140,7 @@ class AddCommand(BaseCommand):
                 run_pip(project, pip_args)
         except subprocess.CalledProcessError as e:
             project.core.ui.echo(
-                "[error]Installation failed:[/]\n" + e.output.decode("utf8"), err=True
+                "[error]Installation failed:[/]\n" + e.output, err=True
             )
             sys.exit(1)
         else:
@@ -210,7 +219,7 @@ class RemoveCommand(BaseCommand):
                 run_pip(project, pip_args)
         except subprocess.CalledProcessError as e:
             project.core.ui.echo(
-                "[error]Uninstallation failed:[/]\n" + e.output.decode("utf8"), err=True
+                "[error]Uninstallation failed:[/]\n" + e.output, err=True
             )
             sys.exit(1)
         else:
@@ -265,7 +274,7 @@ class UpdateCommand(BaseCommand):
         except subprocess.CalledProcessError as e:
             project.core.ui.echo(
                 f"[error]Installing version [primary]{version}[/] failed:[/]\n"
-                + e.output.decode("utf8"),
+                + e.output,
                 err=True,
             )
             sys.exit(1)
