@@ -4,15 +4,16 @@ import collections
 import dataclasses
 import os
 from pathlib import Path
-from typing import Any, Callable, Iterator, Mapping, MutableMapping, TypeVar
+from typing import Any, Callable, Iterator, Mapping, MutableMapping, cast
 
 import platformdirs
+import rich.theme
 import tomlkit
 
 from pdm import termui
+from pdm._types import Source
 from pdm.exceptions import NoConfigError, PdmUsageError
 
-T = TypeVar("T")
 ui = termui.UI()
 
 REPOSITORY = "repository"
@@ -23,13 +24,16 @@ class RepositoryConfig:
     url: str
     username: str | None = None
     password: str | None = None
+    ca_certs: str | None = None
 
     def __rich__(self) -> str:
-        lines = [f"[cyan]url[/] = {self.url}"]
+        lines = [f"[primary]url[/] = {self.url}"]
         if self.username:
-            lines.append(f"[cyan]username[/] = {self.username}")
+            lines.append(f"[primary]username[/] = {self.username}")
         if self.password:
-            lines.append("[cyan]password[/] = <hidden>")
+            lines.append("[primary]password[/] = <hidden>")
+        if self.ca_certs:
+            lines.append(f"[primary]ca_certs[/] = {self.ca_certs}")
         return "\n".join(lines)
 
 
@@ -46,9 +50,11 @@ def load_config(file_path: Path) -> dict[str, Any]:
     """
 
     def get_item(sub_data: Mapping[str, Any]) -> Mapping[str, Any]:
-        result = {}
+        result: dict[str, Any] = {}
         for k, v in sub_data.items():
-            if k != REPOSITORY and isinstance(v, Mapping):
+            if k == "pypi":
+                result.update((f"{k}.{sub_k}", sub_v) for sub_k, sub_v in v.items())
+            elif k != REPOSITORY and isinstance(v, Mapping):
                 result.update(
                     {f"{k}.{sub_k}": sub_v for sub_k, sub_v in get_item(v).items()}
                 )
@@ -126,14 +132,12 @@ class Config(MutableMapping[str, str]):
             False,
             True,
             coerce=ensure_boolean,
-            replace="auto_global",
         ),
         "global_project.fallback_verbose": ConfigItem(
             "If True show message when global project is used implicitly",
             True,
             True,
             coerce=ensure_boolean,
-            replace=None,
         ),
         "global_project.path": ConfigItem(
             "The path to the global project",
@@ -167,18 +171,15 @@ class Config(MutableMapping[str, str]):
             True,
             env_var="PDM_INSTALL_PARALLEL",
             coerce=ensure_boolean,
-            replace="parallel_install",
         ),
         "install.cache": ConfigItem(
             "Cache wheel installation and only put symlinks in the library root",
             False,
             coerce=ensure_boolean,
-            replace="feature.install_cache",
         ),
         "install.cache_method": ConfigItem(
             "`symlink` or `pth` to create links to the cached installation",
             "symlink",
-            replace="feature.install_cache_method",
         ),
         "python.path": ConfigItem("The Python interpreter path", env_var="PDM_PYTHON"),
         "python.use_pyenv": ConfigItem(
@@ -189,7 +190,6 @@ class Config(MutableMapping[str, str]):
             True,
             env_var="PDM_USE_VENV",
             coerce=ensure_boolean,
-            replace="use_venv",
         ),
         "pypi.url": ConfigItem(
             "The URL of PyPI mirror, defaults to https://pypi.org/simple",
@@ -199,10 +199,22 @@ class Config(MutableMapping[str, str]):
         "pypi.verify_ssl": ConfigItem(
             "Verify SSL certificate when query PyPI", True, coerce=ensure_boolean
         ),
+        "pypi.username": ConfigItem(
+            "The username to access PyPI", env_var="PDM_PYPI_USERNAME"
+        ),
+        "pypi.password": ConfigItem(
+            "The password to access PyPI", env_var="PDM_PYPI_PASSWORD"
+        ),
         "pypi.ca_certs": ConfigItem(
             "Path to a CA certificate bundle used for verifying the identity "
             "of the PyPI server",
         ),
+        "pypi.ignore_stored_index": ConfigItem(
+            "Ignore the configured indexes",
+            False,
+            env_var="PDM_IGNORE_STORED_INDEX",
+            coerce=ensure_boolean,
+        ),
         "pypi.client_cert": ConfigItem(
             "Path to client certificate file, or combined cert/key file",
         ),
@@ -231,7 +243,24 @@ class Config(MutableMapping[str, str]):
             env_var="PDM_VENV_IN_PROJECT",
             coerce=ensure_boolean,
         ),
+        "venv.prompt": ConfigItem(
+            "Define a custom template to be displayed in the prompt when virtualenv is"
+            "active. Variables `project_name` and `python_version` are available for"
+            "formatting",
+            default="{project_name}-{python_version}",
+            env_var="PDM_VENV_PROMPT",
+        ),
+        "venv.with_pip": ConfigItem(
+            "Install pip when creating a new venv",
+            default=False,
+            env_var="PDM_VENV_WITH_PIP",
+            coerce=ensure_boolean,
+        ),
     }
+    _config_map.update(
+        (f"theme.{k}", ConfigItem(f"Theme color for {k}", default=v, global_only=True))
+        for k, v in termui.DEFAULT_THEME.items()
+    )
 
     site: Config | None = None
 
@@ -259,10 +288,22 @@ class Config(MutableMapping[str, str]):
             self._file_data, self.get_defaults() if is_global else {}
         )
 
+    def load_theme(self) -> rich.theme.Theme:
+        if not self.is_global:  # pragma: no cover
+            raise PdmUsageError("Theme can only be loaded from global config")
+        return rich.theme.Theme(
+            {k[6:]: v for k, v in self.items() if k.startswith("theme.")}
+        )
+
     @property
     def self_data(self) -> dict[str, Any]:
         return dict(self._file_data)
 
+    def iter_sources(self) -> Iterator[Source]:
+        for name, data in self._data.items():
+            if name.startswith("pypi.") and name not in self._config_map:
+                yield cast(Source, dict(data, name=name))
+
     def _save_config(self) -> None:
         """Save the changed to config file."""
         self.config_file.parent.mkdir(parents=True, exist_ok=True)
@@ -292,6 +333,12 @@ class Config(MutableMapping[str, str]):
             if len(parts) >= 3 and parts[2] == "password" and value:
                 return "<hidden>"
             return value
+        elif parts[0] == "pypi" and key not in self._config_map:
+            index_key = ".".join(parts[:2])
+            if index_key not in self._data:
+                raise KeyError(f"No PyPI index named {parts[1]}")
+            source = self._data[index_key]
+            return source[parts[2]] if len(parts) >= 3 else source
 
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
@@ -314,13 +361,20 @@ class Config(MutableMapping[str, str]):
         if parts[0] == REPOSITORY:
             if len(parts) < 3:
                 raise PdmUsageError(
-                    "Set repository config with [green]repository.{name}.{attr}"
+                    "Set repository config with [success]repository.{name}.{attr}"
                 )
             self._file_data.setdefault(parts[0], {}).setdefault(
                 parts[1], {}
             ).setdefault(parts[2], value)
             self._save_config()
             return
+        if parts[0] == "pypi" and key not in self._config_map:
+            if len(parts) < 3:
+                raise PdmUsageError("Set index config with [success]pypi.{name}.{attr}")
+            index_key = ".".join(parts[:2])
+            self._file_data.setdefault(index_key, {})[parts[2]] = value
+            self._save_config()
+            return
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
@@ -336,7 +390,7 @@ class Config(MutableMapping[str, str]):
             ui.echo(
                 "WARNING: the config is shadowed by env var '{}', "
                 "the value set won't take effect.".format(env_var),
-                style="yellow",
+                style="warning",
             )
         self._file_data[config_key] = value
         if config.replace:
@@ -349,10 +403,10 @@ class Config(MutableMapping[str, str]):
     def __iter__(self) -> Iterator[str]:
         keys: set[str] = set()
         for key in self._data:
-            if key in self._config_map:
-                keys.add(key)
-            elif key in self.deprecated:
+            if key in self.deprecated:
                 keys.add(self.deprecated[key])
+            elif key != REPOSITORY:
+                keys.add(key)
         return iter(keys)
 
     def __delitem__(self, key: str) -> None:
@@ -366,6 +420,16 @@ class Config(MutableMapping[str, str]):
                 del self._file_data.get(REPOSITORY, {})[parts[1]]
             self._save_config()
             return
+        if parts[0] == "pypi" and key not in self._config_map:
+            if len(parts) < 2:
+                raise PdmUsageError("Should specify the name of index")
+            if len(parts) >= 3:
+                index_key, attr = key.rsplit(".", 1)
+                del self._file_data.get(index_key, {})[attr]
+            else:
+                del self._file_data[key]
+            self._save_config()
+            return
         config_key = self.deprecated.get(key, key)
         config = self._config_map[config_key]
         self._file_data.pop(config_key, None)
@@ -377,14 +441,14 @@ class Config(MutableMapping[str, str]):
             ui.echo(
                 "WARNING: the config is shadowed by env var '{}', "
                 "set value won't take effect.".format(env_var),
-                style="yellow",
+                style="warning",
             )
         self._save_config()
 
     def get_repository_config(self, name_or_url: str) -> RepositoryConfig | None:
         """Get a repository by name or url."""
         if not self.is_global:  # pragma: no cover
-            raise PdmUsageError("No repository config in project config.")
+            raise NoConfigError("repository")
         repositories: Mapping[str, Mapping[str, str | None]] = self._data.get(
             REPOSITORY, {}
         )
